@@ -30,21 +30,13 @@ namespace Tomboy
 
 			// Set extra editor drag targets supported (in addition
 			// to the default TextView's various text formats)...
-			//// Gtk.TargetList list = Gtk.Drag.DestGetTargetList (this);
-
-			IntPtr list_ptr = gtk_drag_dest_get_target_list (this.Handle);
-			Gtk.TargetList list = new Gtk.TargetList (list_ptr);
-
+			Gtk.TargetList list = Gtk.Drag.DestGetTargetList (this);
 			list.Add (Gdk.Atom.Intern ("text/uri-list", false), 0, 1);
 			list.Add (Gdk.Atom.Intern ("_NETSCAPE_URL", false), 0, 1);
+			
+			KeyPressEvent += KeyPressed;
 		}
 
-		// FIXME: Gtk# broke compatibility at some point with the
-		// methodref for DestGetTargetList.  We invoke it manually so
-		// our binary will work for everyone.
-		[DllImport("libgtk-win32-2.0-0.dll")]
-		static extern IntPtr gtk_drag_dest_get_target_list (IntPtr raw);
-
 		public static int DefaultMargin
 		{
 			get { return 8; }
@@ -130,6 +122,47 @@ namespace Tomboy
 				base.OnDragDataReceived (context, x, y, selection_data, info, time);
 			}
 		}
+		
+		[GLib.ConnectBefore()]
+		void KeyPressed (object sender, Gtk.KeyPressEventArgs args)
+		{
+			args.RetVal = true;
+			bool ret_value = false;
+			
+			switch (args.Event.Key)
+			{
+			case Gdk.Key.Return:
+				ret_value = ((NoteBuffer) Buffer).AddNewline ();
+				ScrollMarkOnscreen (Buffer.InsertMark);
+				break;
+			case Gdk.Key.Tab:
+				ret_value = ((NoteBuffer) Buffer).AddTab ();
+				ScrollMarkOnscreen (Buffer.InsertMark);
+				break;
+			case Gdk.Key.ISO_Left_Tab:
+				ret_value = ((NoteBuffer) Buffer).RemoveTab ();
+				ScrollMarkOnscreen (Buffer.InsertMark);
+				break;
+			case Gdk.Key.Delete:
+				ret_value = ((NoteBuffer) Buffer).DeleteKeyHandler ();
+				ScrollMarkOnscreen (Buffer.InsertMark);
+				break;
+			case Gdk.Key.BackSpace:
+				ret_value = ((NoteBuffer) Buffer).BackspaceKeyHandler ();
+				break;
+ 			case Gdk.Key.Left:
+ 			case Gdk.Key.Right:
+ 			case Gdk.Key.Up:
+ 			case Gdk.Key.Down:
+				ret_value = false;
+				break;
+			default:
+				((NoteBuffer) Buffer).CheckSelection ();
+				break;
+			}
+			
+			args.RetVal = ret_value;
+		}
 	}
 
 	public class NoteWindow : ForcedPresentWindow 
@@ -166,6 +199,27 @@ namespace Tomboy
 			AddAccelGroup (accel_group);
 
 			text_menu = new NoteTextMenu (accel_group, note.Buffer, note.Buffer.Undoer);
+
+			// Add the Find menu item to the toolbar Text menu.  It
+			// should only show up in the toplevel Text menu, since
+			// the context menu already has a Find submenu.
+
+			Gtk.SeparatorMenuItem spacer = new Gtk.SeparatorMenuItem ();
+			spacer.Show ();
+			text_menu.Append (spacer);
+
+			Gtk.ImageMenuItem find_item = 
+				new Gtk.ImageMenuItem (Catalog.GetString("Find in This Note"));
+			find_item.Image = new Gtk.Image (Gtk.Stock.Find, Gtk.IconSize.Menu);
+			find_item.Activated += FindActivate;
+			find_item.AddAccelerator ("activate",
+						  accel_group,
+						  (uint) Gdk.Key.f, 
+						  Gdk.ModifierType.ControlMask,
+						  Gtk.AccelFlags.Visible);
+			find_item.Show ();
+			text_menu.Append (find_item);
+
 			plugin_menu = MakePluginMenu ();
 
 			toolbar = MakeToolbar ();
@@ -195,6 +249,7 @@ namespace Tomboy
 			find_bar = new NoteFindBar (note);
 			find_bar.Visible = false;
 			find_bar.NoShowAll = true;
+			find_bar.Hidden += FindBarHidden;
 
 			Gtk.VBox box = new Gtk.VBox (false, 2);
 			box.PackStart (toolbar, false, false, 0);
@@ -223,12 +278,6 @@ namespace Tomboy
 						    Gdk.ModifierType.ControlMask,
 						    Gtk.AccelFlags.Visible);
 
-			// Open Find Dialog (Ctrl-F)
-			global_keys.AddAccelerator (new EventHandler (FindActivate),
-						    (uint) Gdk.Key.f, 
-						    Gdk.ModifierType.ControlMask,
-						    Gtk.AccelFlags.Visible);
-
 			// Find Next (Ctrl-G)
 			global_keys.AddAccelerator (new EventHandler (FindNextActivate),
 						    (uint) Gdk.Key.g, 
@@ -248,9 +297,27 @@ namespace Tomboy
 						    0,
 						    0);
 			
+			// Create a new note
+			global_keys.AddAccelerator (new EventHandler (CreateNewNote),
+							(uint) Gdk.Key.n,
+							Gdk.ModifierType.ControlMask,
+							Gtk.AccelFlags.Visible);
+			
 			// Have Esc key close the note window
 			KeyPressEvent += KeyPressed;
+						   
+			// Increase Indent			    
+			global_keys.AddAccelerator (new EventHandler (IncreaseDepthHandler),
+						    (uint) Gdk.Key.Right, 
+						    Gdk.ModifierType.Mod1Mask,
+						    Gtk.AccelFlags.Visible);
 						    
+			// Decrease Indent
+			global_keys.AddAccelerator (new EventHandler (DecreaseDepthHandler),
+						    (uint) Gdk.Key.Left, 
+						    Gdk.ModifierType.Mod1Mask,
+						    Gtk.AccelFlags.Visible);			
+
 			this.Add (box);
 		}
 
@@ -388,6 +455,18 @@ namespace Tomboy
 			Gtk.MenuItem spacer1 = new Gtk.SeparatorMenuItem ();
 			spacer1.Show ();
 
+			Gtk.ImageMenuItem search = new Gtk.ImageMenuItem (
+				Catalog.GetString ("_Search All Notes"));
+			search.Image = new Gtk.Image (Gtk.Stock.Find, Gtk.IconSize.Menu);
+			search.Activated += SearchActivate;
+			search.AddAccelerator ("activate",
+					       accel_group,
+					       (uint) Gdk.Key.f, 
+					       (Gdk.ModifierType.ControlMask | 
+						Gdk.ModifierType.ShiftMask),
+					       Gtk.AccelFlags.Visible);
+			search.Show ();
+
 			Gtk.ImageMenuItem link = 
 				new Gtk.ImageMenuItem (Catalog.GetString ("_Link to New Note"));
 			link.Image = new Gtk.Image (Gtk.Stock.JumpTo, Gtk.IconSize.Menu);
@@ -400,17 +479,6 @@ namespace Tomboy
 					     Gtk.AccelFlags.Visible);
 			link.Show ();
 
-			Gtk.ImageMenuItem toc = new Gtk.ImageMenuItem (
-				Catalog.GetString ("_Open Table of Contents"));
-			toc.Image = new Gtk.Image (Gtk.Stock.SortAscending, Gtk.IconSize.Menu);
-			toc.Activated += OpenTOCActivate;
-			toc.AddAccelerator ("activate",
-					     accel_group,
-					     (uint) Gdk.Key.o, 
-					     Gdk.ModifierType.ControlMask,
-					     Gtk.AccelFlags.Visible);
-			toc.Show ();
-
 			Gtk.ImageMenuItem text_item = 
 				new Gtk.ImageMenuItem (Catalog.GetString ("Te_xt"));
 			text_item.Image = new Gtk.Image (Gtk.Stock.SelectFont, Gtk.IconSize.Menu);
@@ -420,7 +488,7 @@ namespace Tomboy
 			text_item.Show ();
 
 			Gtk.ImageMenuItem find_item = 
-				new Gtk.ImageMenuItem (Catalog.GetString ("_Find"));
+				new Gtk.ImageMenuItem (Catalog.GetString ("_Find in This Note"));
 			find_item.Image = new Gtk.Image (Gtk.Stock.Find, Gtk.IconSize.Menu);
 			find_item.Submenu = MakeFindMenu ();
 			find_item.Show ();
@@ -431,8 +499,8 @@ namespace Tomboy
 			args.Menu.Prepend (spacer1);
 			args.Menu.Prepend (text_item);
 			args.Menu.Prepend (find_item);
-			args.Menu.Prepend (toc);
 			args.Menu.Prepend (link);
+			args.Menu.Prepend (search);
 
 			Gtk.MenuItem close_all = 
 				new Gtk.MenuItem (Catalog.GetString ("Clos_e All Notes"));
@@ -471,23 +539,25 @@ namespace Tomboy
 			Gtk.Toolbar toolbar = new Gtk.Toolbar ();
 			toolbar.Tooltips = true;
 
-			Gtk.Widget find = 
+			Gtk.Widget search = 
 				toolbar.AppendItem (
-					Catalog.GetString ("Find"), 
-					Catalog.GetString ("Find in this note"),
+					Catalog.GetString ("Search"), 
+					Catalog.GetString ("Search your notes (Ctrl-Shift-F)"),
 					null, 
 					new Gtk.Image (Gtk.Stock.Find, toolbar.IconSize),
-					new Gtk.SignalFunc (FindButtonClicked));
-			find.AddAccelerator ("activate",
-					     accel_group,
-					     (uint) Gdk.Key.f, 
-					     Gdk.ModifierType.ControlMask,
-					     Gtk.AccelFlags.Visible);
+					new Gtk.SignalFunc (SearchButtonClicked));
+			search.AddAccelerator ("activate",
+					       accel_group,
+					       (uint) Gdk.Key.f, 
+					       (Gdk.ModifierType.ControlMask | 
+						Gdk.ModifierType.ShiftMask),
+					       Gtk.AccelFlags.Visible);
 
 			link_button = 
 				toolbar.AppendItem (
 					Catalog.GetString ("Link"), 
-					Catalog.GetString ("Link selected text to a new note"), 
+					Catalog.GetString (
+						"Link selected text to a new note (Ctrl-L)"), 
 					null, 
 					new Gtk.Image (Gtk.Stock.JumpTo, toolbar.IconSize),
 					new Gtk.SignalFunc (LinkButtonClicked));
@@ -521,21 +591,6 @@ namespace Tomboy
 
 			toolbar.AppendSpace ();
 
-			Gtk.Widget toc = 
-				toolbar.AppendItem (
-					Catalog.GetString ("ToC"), 
-					Catalog.GetString ("Open Table of Contents"),
-					null, 
-					new Gtk.Image (Gtk.Stock.SortAscending, toolbar.IconSize),
-					new Gtk.SignalFunc (TOCButtonClicked));
-			toc.AddAccelerator ("activate",
-					    accel_group,
-					    (uint) Gdk.Key.o, 
-					    Gdk.ModifierType.ControlMask,
-					    Gtk.AccelFlags.Visible);
-
-			toolbar.AppendSpace ();
-
 		        Gtk.Widget delete = 
 				toolbar.AppendItem (
 					Catalog.GetString ("Delete"), 
@@ -657,6 +712,11 @@ namespace Tomboy
 			Find.SearchText = note.Buffer.Selection;
 		}
 
+		void FindActivate (object sender, EventArgs args)
+		{
+			FindButtonClicked ();
+		}
+
 		void FindNextActivate (object sender, EventArgs args)
 		{
 			Find.FindNextButton.Click ();
@@ -666,14 +726,12 @@ namespace Tomboy
 		{
 			Find.FindPreviousButton.Click ();
 		}
-
-		// 
-		// Signature trampoline for editor context-menu "Find"
-		//
-
-		void FindActivate (object sender, EventArgs args)
+		
+		void FindBarHidden (object sender, EventArgs args)
 		{
-			FindButtonClicked ();
+			// Reposition the current focus back to the editor so the
+			// cursor will be ready for typing.
+			editor.GrabFocus ();
 		}
 
 		//
@@ -716,10 +774,6 @@ namespace Tomboy
 			match.Window.Present ();
 		}
 
-		// 
-		// Signature trampoline for editor context-menu "Link to Note"
-		//
-
 		void LinkToNoteActivate (object sender, EventArgs args)
 		{
 			LinkButtonClicked ();
@@ -727,20 +781,37 @@ namespace Tomboy
 
 		void OpenHelpActivate (object sender, EventArgs args)
 		{
-			GuiUtils.ShowHelp("tomboy.xml", "editing-notes", Screen, this);
+			GuiUtils.ShowHelp ("tomboy.xml", "editing-notes", Screen, this);
+		}
+		
+		void CreateNewNote (object sender, EventArgs args)
+		{
+			Tomboy.ActionManager ["NewNoteAction"].Activate ();
 		}
 
-		void TOCButtonClicked ()
+		void SearchButtonClicked ()
 		{
-			NoteRecentChanges toc = NoteRecentChanges.GetInstance (note.Manager);
-			toc.SearchText = note.Buffer.Selection;
-			toc.Present ();
+			NoteRecentChanges search = NoteRecentChanges.GetInstance (note.Manager);
+			if (note.Buffer.Selection != null) {
+				search.SearchText = note.Buffer.Selection;
+			}
+			search.Present ();
 		}
 
-		void OpenTOCActivate (object sender, EventArgs args)
+		void SearchActivate (object sender, EventArgs args)
 		{
-			TOCButtonClicked ();
+			SearchButtonClicked ();
 		}
+
+		void IncreaseDepthHandler (object sender, EventArgs args)
+		{
+			((NoteBuffer)editor.Buffer).IncreaseCursorDepth();
+		}
+		
+		void DecreaseDepthHandler (object sender, EventArgs args)
+		{
+			((NoteBuffer)editor.Buffer).DecreaseCursorDepth();
+		}	
 	}
 	
 	public class NoteFindBar : Gtk.HBox
@@ -756,6 +827,7 @@ namespace Tomboy
 		string prev_search_text;
 		
 		InterruptableTimeout entry_changed_timeout;
+		InterruptableTimeout note_changed_timeout;
 
 		bool shift_key_pressed;
 		
@@ -763,9 +835,6 @@ namespace Tomboy
 		{
 			this.note = note;
 			
-			note.Buffer.InsertText += OnInsertText;
-			note.Buffer.DeleteRange += OnDeleteRange;
-			
 			BorderWidth = 2;
 			
 			Gtk.Button button = new Gtk.Button ();
@@ -790,6 +859,7 @@ namespace Tomboy
 			prev_button.Image = new Gtk.Image (Gtk.Stock.GoBack, Gtk.IconSize.Menu);
 			prev_button.Relief = Gtk.ReliefStyle.None;
 			prev_button.Sensitive = false;
+			prev_button.FocusOnClick = false;
 			prev_button.Clicked += OnPrevClicked;
 			prev_button.Show ();
 			PackStart (prev_button, false, false, 0);
@@ -798,6 +868,7 @@ namespace Tomboy
 			next_button.Image = new Gtk.Image (Gtk.Stock.GoForward, Gtk.IconSize.Menu);
 			next_button.Relief = Gtk.ReliefStyle.None;
 			next_button.Sensitive = false;
+			next_button.FocusOnClick = false;
 			next_button.Clicked += OnNextClicked;
 			next_button.Show ();
 			PackStart (next_button, false, false, 0);
@@ -823,6 +894,11 @@ namespace Tomboy
 
 			// Highlight words from a previous existing search
 			HighlightMatches (true);
+			
+			// Call PerformSearch on newly inserted text when
+			// the FindBar is visible
+			note.Buffer.InsertText += OnInsertText;
+			note.Buffer.DeleteRange += OnDeleteRange;
 
 			base.OnShown ();
 		}
@@ -830,6 +906,10 @@ namespace Tomboy
 		protected override void OnHidden ()
 		{
 			HighlightMatches (false);
+			
+			// Prevent searching when the FindBar is not visible
+			note.Buffer.InsertText -= OnInsertText;
+			note.Buffer.DeleteRange -= OnDeleteRange;
 
 			base.OnShown ();
 		}
@@ -900,20 +980,22 @@ namespace Tomboy
 
 		void OnCaseSensitiveToggled (object sender, EventArgs args)
 		{
-			PerformSearch ();
+			PerformSearch (true);
 		}
 		
 		void OnFindEntryActivated (object sender, EventArgs args)
 		{
-			if (entry_changed_timeout != null)
+			if (entry_changed_timeout != null) {
 				entry_changed_timeout.Cancel ();
+				entry_changed_timeout = null;
+			}
 			
-			if (prev_search_text != null 
-				&& SearchText != null 
-				&& prev_search_text.CompareTo (SearchText) == 0)
+			if (prev_search_text != null && 
+			    SearchText != null && 
+			    prev_search_text.CompareTo (SearchText) == 0)
 				next_button.Click ();
 			else
-				PerformSearch ();
+				PerformSearch (true);
 		}
 		
 		void OnFindEntryChanged (object sender, EventArgs args)
@@ -924,23 +1006,25 @@ namespace Tomboy
 			}
 			
 			if (SearchText == null) {
-				PerformSearch ();
+				PerformSearch (false);
 			} else {
 				entry_changed_timeout.Reset (500);
 			}
 		}
 		
 		// Called after .5 seconds of typing inactivity, or on explicit
-		// activate.  Redo the serach and update the results...
+		// activate.  Redo the search and update the results...
 		void EntryChangedTimeout (object sender, EventArgs args)
 		{
+			entry_changed_timeout = null;
+			
 			if (SearchText == null)
 				return;
 			
-			PerformSearch ();
+			PerformSearch (true);
 		}
 		
-		void PerformSearch ()
+		void PerformSearch (bool scroll_to_hit)
 		{
 			CleanupMatches ();
 
@@ -953,9 +1037,6 @@ namespace Tomboy
 
 			string [] words = text.Split (' ', '\t', '\n');
 
-			// Used for matching in the raw note XML
-			string [] encoded_words = XmlEncoder.Encode (text).Split (' ', '\t', '\n');
-
 			current_matches = 
 				FindMatchesInBuffer (note.Buffer, 
 						     words, 
@@ -963,8 +1044,13 @@ namespace Tomboy
 			
 			prev_search_text = SearchText;
 
-			if (current_matches != null)
+			if (current_matches != null) {
 				HighlightMatches (true);
+				
+				// Select/scroll to the first match
+				if (scroll_to_hit)
+					OnNextClicked (this, EventArgs.Empty);
+			}
 
 			UpdateSensitivity ();
 		}
@@ -985,14 +1071,42 @@ namespace Tomboy
 			}
 		}
 		
+		void UpdateSearch ()
+		{
+			if (note_changed_timeout == null) {
+				note_changed_timeout = new InterruptableTimeout ();
+				note_changed_timeout.Timeout += NoteChangedTimeout;
+			}
+			
+			if (SearchText == null) {
+				PerformSearch (false);
+			} else {
+				note_changed_timeout.Reset (500);
+			}
+		}
+		
+		// Called after .5 seconds of typing inactivity to update
+		// the search when the text of a note changes.  This prevents
+		// the search from running on every single change made in a
+		// note.
+		void NoteChangedTimeout (object sender, EventArgs args)
+		{
+			note_changed_timeout = null;
+			
+			if (SearchText == null)
+				return;
+			
+			PerformSearch (false);
+		}
+		
 		void OnInsertText (object sender, Gtk.InsertTextArgs args)
 		{
-			PerformSearch ();
+			UpdateSearch ();
 		}
 		
 		void OnDeleteRange (object sender, Gtk.DeleteRangeArgs args)
 		{
-			PerformSearch ();
+			UpdateSearch ();
 		}
 		
 		//
@@ -1177,6 +1291,9 @@ namespace Tomboy
 		Gtk.RadioMenuItem large;
 		Gtk.RadioMenuItem small;
 		Gtk.CheckMenuItem highlight;
+		Gtk.CheckMenuItem bullets;
+		Gtk.ImageMenuItem increase_indent;
+		Gtk.ImageMenuItem decrease_indent;
 
 		// Active when the text size is indeterminable, such as when in
 		// the note's title line.
@@ -1316,7 +1433,20 @@ namespace Tomboy
 
 			hidden_no_size = new Gtk.RadioMenuItem (small.Group, string.Empty);
 			hidden_no_size.Hide ();
+			
+			Gtk.SeparatorMenuItem spacer2 = new Gtk.SeparatorMenuItem ();
+			
+			bullets = new Gtk.CheckMenuItem (Catalog.GetString ("Bullets"));
+			bullets.Activated += ToggleBulletsClicked;
 
+			increase_indent = new Gtk.ImageMenuItem (Gtk.Stock.Indent, accel_group);
+			increase_indent.Activated += IncreaseIndentClicked;
+			increase_indent.Show ();
+			
+			decrease_indent = new Gtk.ImageMenuItem (Gtk.Stock.Unindent, accel_group);
+			decrease_indent.Activated += DecreaseIndentClicked;
+			decrease_indent.Show ();
+			
 			RefreshState ();
 
 			Append (bold);
@@ -1329,6 +1459,10 @@ namespace Tomboy
 			Append (normal);
 			Append (large);
 			Append (huge);
+			Append (spacer2);
+			Append (bullets);
+			Append (increase_indent);
+			Append (decrease_indent);
 			ShowAll ();
 		}
 
@@ -1373,6 +1507,13 @@ namespace Tomboy
 			italic.Active = buffer.IsActiveTag ("italic");
 			strikeout.Active = buffer.IsActiveTag ("strikethrough");
 			highlight.Active = buffer.IsActiveTag ("highlight");
+			
+			bool inside_bullets = buffer.IsBulletedListActive ();
+			bullets.Activated -= ToggleBulletsClicked;
+			bullets.Active = inside_bullets;
+			bullets.Activated += ToggleBulletsClicked;
+			increase_indent.Sensitive = inside_bullets;
+			decrease_indent.Sensitive = inside_bullets;
 
 			RefreshSizingState ();
 
@@ -1449,5 +1590,36 @@ namespace Tomboy
 			undo.Sensitive = undo_manager.CanUndo;
 			redo.Sensitive = undo_manager.CanRedo;
 		}
+		
+		//
+		// Bulleted list handlers
+		//
+		void ToggleBulletsClicked (object sender, EventArgs args)
+		{
+			bool inside_bullets = buffer.IsBulletedListActive ();
+			if (!inside_bullets)
+				buffer.IncreaseCursorDepth ();
+			else {
+				// FIXME: This is a bad hack.  There's got to be
+				// a better way to remove the bullet.
+				
+				// For now, keep calling buffer.DecreaseCursorDepth ()
+				// until bullets are no longer present.
+				do {
+					buffer.DecreaseCursorDepth ();
+					inside_bullets = buffer.IsBulletedListActive ();
+				} while (inside_bullets == true);
+			}
+		}
+		
+		void IncreaseIndentClicked (object sender, EventArgs args)
+		{
+			buffer.IncreaseCursorDepth ();
+		}
+		
+		void DecreaseIndentClicked (object sender, EventArgs args)
+		{
+			buffer.DecreaseCursorDepth ();
+		}
 	}
 }
